# Copyright (c) 2009-2015 testtools developers. See LICENSE for details.

"""Test ConcurrentTestSuite and related things."""

import doctest
from pprint import pformat
import unittest

from testtools import (
    ConcurrentTestSuite,
    ConcurrentStreamTestSuite,
    iterate_tests,
    PlaceHolder,
    TestByTestResult,
    TestCase,
)
from testtools.helpers import try_import
from testtools.matchers import DocTestMatches, Equals
from testtools.testresult.doubles import StreamResult as LoggingStream
from testtools.testsuite import FixtureSuite, sorted_tests
from testtools.tests.helpers import LoggingResult

FunctionFixture = try_import("fixtures.FunctionFixture")


class Sample(TestCase):
    def __hash__(self):
        return id(self)

    def test_method1(self):
        pass

    def test_method2(self):
        pass


class TestConcurrentTestSuiteRun(TestCase):
    def test_broken_test(self):
        log = []

        def on_test(test, status, start_time, stop_time, tags, details):
            log.append((test.id(), status, set(details.keys())))

        class BrokenTest:
            # Simple break - no result parameter to run()
            def __call__(self):
                pass

            run = __call__

        original_suite = unittest.TestSuite([BrokenTest()])
        suite = ConcurrentTestSuite(original_suite, self.split_suite)
        suite.run(TestByTestResult(on_test))
        self.assertEqual([("broken-runner", "error", {"traceback"})], log)

    def test_trivial(self):
        log = []
        result = LoggingResult(log)
        test1 = Sample("test_method1")
        test2 = Sample("test_method2")
        original_suite = unittest.TestSuite([test1, test2])
        suite = ConcurrentTestSuite(original_suite, self.split_suite)
        suite.run(result)
        # log[0] is the timestamp for the first test starting.
        test1 = log[1][1]
        test2 = log[-1][1]
        self.assertIsInstance(test1, Sample)
        self.assertIsInstance(test2, Sample)
        self.assertNotEqual(test1.id(), test2.id())

    def test_wrap_result(self):
        # ConcurrentTestSuite has a hook for wrapping the per-thread result.
        wrap_log = []

        def wrap_result(thread_safe_result, thread_number):
            wrap_log.append((thread_safe_result.result.decorated, thread_number))
            return thread_safe_result

        result_log = []
        result = LoggingResult(result_log)
        test1 = Sample("test_method1")
        test2 = Sample("test_method2")
        original_suite = unittest.TestSuite([test1, test2])
        suite = ConcurrentTestSuite(
            original_suite, self.split_suite, wrap_result=wrap_result
        )
        suite.run(result)
        self.assertEqual(
            [
                (result, 0),
                (result, 1),
            ],
            wrap_log,
        )
        # Smoke test to make sure everything ran OK.
        self.assertNotEqual([], result_log)

    def split_suite(self, suite):
        return list(iterate_tests(suite))


class TestConcurrentStreamTestSuiteRun(TestCase):
    def test_trivial(self):
        result = LoggingStream()
        test1 = Sample("test_method1")
        test2 = Sample("test_method2")

        def cases():
            return [(test1, "0"), (test2, "1")]

        suite = ConcurrentStreamTestSuite(cases)
        suite.run(result)

        def freeze(set_or_none):
            if set_or_none is None:
                return set_or_none
            return frozenset(set_or_none)

        # Ignore event order: we're testing the code is all glued together,
        # which just means we can pump events through and they get route codes
        # added appropriately.
        self.assertEqual(
            {
                (
                    "status",
                    "testtools.tests.test_testsuite.Sample.test_method1",
                    "inprogress",
                    None,
                    True,
                    None,
                    None,
                    False,
                    None,
                    "0",
                    None,
                ),
                (
                    "status",
                    "testtools.tests.test_testsuite.Sample.test_method1",
                    "success",
                    frozenset(),
                    True,
                    None,
                    None,
                    False,
                    None,
                    "0",
                    None,
                ),
                (
                    "status",
                    "testtools.tests.test_testsuite.Sample.test_method2",
                    "inprogress",
                    None,
                    True,
                    None,
                    None,
                    False,
                    None,
                    "1",
                    None,
                ),
                (
                    "status",
                    "testtools.tests.test_testsuite.Sample.test_method2",
                    "success",
                    frozenset(),
                    True,
                    None,
                    None,
                    False,
                    None,
                    "1",
                    None,
                ),
            },
            {
                event[0:3] + (freeze(event[3]),) + event[4:10] + (None,)
                for event in result._events
            },
        )

    def test_broken_runner(self):
        # If the object called breaks, the stream is informed about it
        # regardless.
        class BrokenTest:
            # broken - no result parameter!
            def __call__(self):
                pass

            def run(self):
                pass

        result = LoggingStream()

        def cases():
            return [(BrokenTest(), "0")]

        suite = ConcurrentStreamTestSuite(cases)
        suite.run(result)
        events = result._events
        # Check the traceback loosely.
        self.assertEqual(
            events[1][6].decode("utf8"), "Traceback (most recent call last):\n"
        )
        self.assertThat(
            events[2][6].decode("utf8"),
            DocTestMatches(
                """\
  File "...testtools/testsuite.py", line ..., in _run_test
    test.run(process_result)...
""",
                doctest.ELLIPSIS,
            ),
        )
        self.assertThat(
            events[3][6].decode("utf8"),
            DocTestMatches(
                """\
TypeError: ...run() takes ...1 ...argument...2...given...
""",
                doctest.ELLIPSIS,
            ),
        )
        events = [event[0:10] + (None,) for event in events]
        events[1] = events[1][:6] + (None,) + events[1][7:]
        events[2] = events[2][:6] + (None,) + events[2][7:]
        events[3] = events[3][:6] + (None,) + events[3][7:]
        self.assertEqual(
            [
                (
                    "status",
                    "broken-runner-'0'",
                    "inprogress",
                    None,
                    True,
                    None,
                    None,
                    False,
                    None,
                    "0",
                    None,
                ),
                (
                    "status",
                    "broken-runner-'0'",
                    None,
                    None,
                    True,
                    "traceback",
                    None,
                    False,
                    'text/x-traceback; charset="utf8"; language="python"',
                    "0",
                    None,
                ),
                (
                    "status",
                    "broken-runner-'0'",
                    None,
                    None,
                    True,
                    "traceback",
                    None,
                    False,
                    'text/x-traceback; charset="utf8"; language="python"',
                    "0",
                    None,
                ),
                (
                    "status",
                    "broken-runner-'0'",
                    None,
                    None,
                    True,
                    "traceback",
                    None,
                    True,
                    'text/x-traceback; charset="utf8"; language="python"',
                    "0",
                    None,
                ),
                (
                    "status",
                    "broken-runner-'0'",
                    "fail",
                    set(),
                    True,
                    None,
                    None,
                    False,
                    None,
                    "0",
                    None,
                ),
            ],
            events,
        )

    def split_suite(self, suite):
        tests = list(enumerate(iterate_tests(suite)))
        return [(test, str(pos)) for pos, test in tests]

    def test_setupclass_skip(self):
        # We should support setupclass skipping using cls.skipException.
        # Because folk have used that.
        class Skips(TestCase):
            @classmethod
            def setUpClass(cls):
                raise cls.skipException("foo")

            def test_notrun(self):
                pass

        # Test discovery uses the default suite from unittest (unless users
        # deliberately change things, in which case they keep both pieces).
        suite = unittest.TestSuite([Skips("test_notrun")])
        log = []
        result = LoggingResult(log)
        suite.run(result)
        self.assertEqual(["addSkip"], [item[0] for item in log])

    def test_setupclass_upcall(self):
        # Note that this is kindof-a-case-test, kindof-suite, because
        # setUpClass is linked between them.
        class Simples(TestCase):
            @classmethod
            def setUpClass(cls):
                super().setUpClass()

            def test_simple(self):
                pass

        # Test discovery uses the default suite from unittest (unless users
        # deliberately change things, in which case they keep both pieces).
        suite = unittest.TestSuite([Simples("test_simple")])
        log = []
        result = LoggingResult(log)
        suite.run(result)
        self.assertEqual(
            ["startTest", "addSuccess", "stopTest"], [item[0] for item in log]
        )


class TestFixtureSuite(TestCase):
    def setUp(self):
        super().setUp()
        if FunctionFixture is None:
            self.skipTest("Need fixtures")

    def test_fixture_suite(self):
        log = []

        class Sample(TestCase):
            def test_one(self):
                log.append(1)

            def test_two(self):
                log.append(2)

        fixture = FunctionFixture(
            lambda: log.append("setUp"), lambda fixture: log.append("tearDown")
        )
        suite = FixtureSuite(fixture, [Sample("test_one"), Sample("test_two")])
        suite.run(LoggingResult([]))
        self.assertEqual(["setUp", 1, 2, "tearDown"], log)

    def test_fixture_suite_sort(self):
        log = []

        class Sample(TestCase):
            def test_one(self):
                log.append(1)

            def test_two(self):
                log.append(2)

        fixture = FunctionFixture(
            lambda: log.append("setUp"), lambda fixture: log.append("tearDown")
        )
        suite = FixtureSuite(fixture, [Sample("test_one"), Sample("test_one")])
        self.assertRaises(ValueError, suite.sort_tests)


class TestSortedTests(TestCase):
    def test_sorts_custom_suites(self):
        a = PlaceHolder("a")
        b = PlaceHolder("b")

        class Subclass(unittest.TestSuite):
            def sort_tests(self):
                self._tests = sorted_tests(self, True)

        input_suite = Subclass([b, a])
        suite = sorted_tests(input_suite)
        self.assertEqual([a, b], list(iterate_tests(suite)))
        self.assertEqual([input_suite], list(iter(suite)))

    def test_custom_suite_without_sort_tests_works(self):
        a = PlaceHolder("a")
        b = PlaceHolder("b")

        class Subclass(unittest.TestSuite):
            pass

        input_suite = Subclass([b, a])
        suite = sorted_tests(input_suite)
        self.assertEqual([b, a], list(iterate_tests(suite)))
        self.assertEqual([input_suite], list(iter(suite)))

    def test_sorts_simple_suites(self):
        a = PlaceHolder("a")
        b = PlaceHolder("b")
        suite = sorted_tests(unittest.TestSuite([b, a]))
        self.assertEqual([a, b], list(iterate_tests(suite)))

    def test_duplicate_simple_suites(self):
        a = PlaceHolder("a")
        b = PlaceHolder("b")
        c = PlaceHolder("a")
        self.assertRaises(ValueError, sorted_tests, unittest.TestSuite([a, b, c]))

    def test_multiple_duplicates(self):
        # If there are multiple duplicates on a test suite, we report on them
        # all.
        a = PlaceHolder("a")
        b = PlaceHolder("b")
        c = PlaceHolder("a")
        d = PlaceHolder("b")
        error = self.assertRaises(
            ValueError, sorted_tests, unittest.TestSuite([a, b, c, d])
        )
        self.assertThat(
            str(error),
            Equals("Duplicate test ids detected: {}".format(pformat({"a": 2, "b": 2}))),
        )


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2010-2011 testtools developers. See LICENSE for details.

import unittest

from testtools import (
    TestCase,
    content,
    content_type,
)
from testtools.compat import _b
from testtools.helpers import try_import
from testtools.matchers import (
    Contains,
    Equals,
)
from testtools.testresult.doubles import (
    ExtendedTestResult,
)

fixtures = try_import("fixtures")
LoggingFixture = try_import("fixtures.tests.helpers.LoggingFixture")


class TestFixtureSupport(TestCase):
    def setUp(self):
        super().setUp()
        if fixtures is None or LoggingFixture is None:
            self.skipTest("Need fixtures")

    def test_useFixture(self):
        fixture = LoggingFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)

        result = unittest.TestResult()
        SimpleTest("test_foo").run(result)
        self.assertTrue(result.wasSuccessful())
        self.assertEqual(["setUp", "cleanUp"], fixture.calls)

    def test_useFixture_cleanups_raise_caught(self):
        calls = []

        def raiser(ignored):
            calls.append("called")
            raise Exception("foo")

        fixture = fixtures.FunctionFixture(lambda: None, raiser)

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)

        result = unittest.TestResult()
        SimpleTest("test_foo").run(result)
        self.assertFalse(result.wasSuccessful())
        self.assertEqual(["called"], calls)

    def test_useFixture_details_captured(self):
        class DetailsFixture(fixtures.Fixture):
            def setUp(self):
                fixtures.Fixture.setUp(self)
                self.addCleanup(delattr, self, "content")
                self.content = [_b("content available until cleanUp")]
                self.addDetail(
                    "content", content.Content(content_type.UTF8_TEXT, self.get_content)
                )

            def get_content(self):
                return self.content

        fixture = DetailsFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)
                # Add a colliding detail (both should show up)
                self.addDetail(
                    "content",
                    content.Content(content_type.UTF8_TEXT, lambda: [_b("foo")]),
                )

        result = ExtendedTestResult()
        SimpleTest("test_foo").run(result)
        self.assertEqual("addSuccess", result._events[-2][0])
        details = result._events[-2][2]
        self.assertEqual(["content", "content-1"], sorted(details.keys()))
        self.assertEqual("foo", details["content"].as_text())
        self.assertEqual(
            "content available until cleanUp", details["content-1"].as_text()
        )

    def test_useFixture_multiple_details_captured(self):
        class DetailsFixture(fixtures.Fixture):
            def setUp(self):
                fixtures.Fixture.setUp(self)
                self.addDetail("aaa", content.text_content("foo"))
                self.addDetail("bbb", content.text_content("bar"))

        fixture = DetailsFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)

        result = ExtendedTestResult()
        SimpleTest("test_foo").run(result)
        self.assertEqual("addSuccess", result._events[-2][0])
        details = result._events[-2][2]
        self.assertEqual(["aaa", "bbb"], sorted(details))
        self.assertEqual("foo", details["aaa"].as_text())
        self.assertEqual("bar", details["bbb"].as_text())

    def test_useFixture_details_captured_from_setUp(self):
        # Details added during fixture set-up are gathered even if setUp()
        # fails with an exception.
        class BrokenFixture(fixtures.Fixture):
            def setUp(self):
                fixtures.Fixture.setUp(self)
                self.addDetail("content", content.text_content("foobar"))
                raise Exception()

        fixture = BrokenFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)

        result = ExtendedTestResult()
        SimpleTest("test_foo").run(result)
        self.assertEqual("addError", result._events[-2][0])
        details = result._events[-2][2]
        self.assertEqual(["content", "traceback"], sorted(details))
        self.assertEqual("foobar", "".join(details["content"].iter_text()))

    def test_useFixture_details_captured_from__setUp(self):
        # Newer Fixtures deprecates setUp() in favour of _setUp().
        # https://bugs.launchpad.net/testtools/+bug/1469759 reports that
        # this is broken when gathering details from a broken _setUp().
        class BrokenFixture(fixtures.Fixture):
            def _setUp(self):
                fixtures.Fixture._setUp(self)
                self.addDetail("broken", content.text_content("foobar"))
                raise Exception("_setUp broke")

        fixture = BrokenFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.addDetail("foo_content", content.text_content("foo ok"))
                self.useFixture(fixture)

        result = ExtendedTestResult()
        SimpleTest("test_foo").run(result)
        self.assertEqual("addError", result._events[-2][0])
        details = result._events[-2][2]
        self.assertEqual(
            ["broken", "foo_content", "traceback", "traceback-1"], sorted(details)
        )
        self.expectThat("".join(details["broken"].iter_text()), Equals("foobar"))
        self.expectThat("".join(details["foo_content"].iter_text()), Equals("foo ok"))
        self.expectThat(
            "".join(details["traceback"].iter_text()), Contains("_setUp broke")
        )
        self.expectThat("".join(details["traceback-1"].iter_text()), Contains("foobar"))

    def test_useFixture_original_exception_raised_if_gather_details_fails(self):
        # In bug #1368440 it was reported that when a fixture fails setUp
        # and gather_details errors on it, then the original exception that
        # failed is not reported.
        class BrokenFixture(fixtures.Fixture):
            def getDetails(self):
                raise AttributeError("getDetails broke")

            def setUp(self):
                fixtures.Fixture.setUp(self)
                raise Exception("setUp broke")

        fixture = BrokenFixture()

        class SimpleTest(TestCase):
            def test_foo(self):
                self.useFixture(fixture)

        result = ExtendedTestResult()
        SimpleTest("test_foo").run(result)
        self.assertEqual("addError", result._events[-2][0])
        details = result._events[-2][2]
        self.assertEqual(["traceback", "traceback-1"], sorted(details))
        self.assertThat(
            "".join(details["traceback"].iter_text()), Contains("setUp broke")
        )
        self.assertThat(
            "".join(details["traceback-1"].iter_text()), Contains("getDetails broke")
        )


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2010-2012 testtools developers. See LICENSE for details.

import sys


def try_import(name, alternative=None, error_callback=None):
    """Attempt to import a module, with a fallback.

    Attempt to import ``name``.  If it fails, return ``alternative``.  When
    supporting multiple versions of Python or optional dependencies, it is
    useful to be able to try to import a module.

    :param name: The name of the object to import, e.g. ``os.path`` or
        ``os.path.join``.
    :param alternative: The value to return if no module can be imported.
        Defaults to None.
    :param error_callback: If non-None, a callable that is passed the
        ImportError when the module cannot be loaded.
    """
    module_segments = name.split(".")
    last_error = None
    remainder = []

    # module_name will be what successfully imports. We cannot walk from the
    # __import__ result because in import loops (A imports A.B, which imports
    # C, which calls try_import("A.B")) A.B will not yet be set.
    while module_segments:
        module_name = ".".join(module_segments)
        try:
            __import__(module_name)
        except ImportError:
            last_error = sys.exc_info()[1]
            remainder.append(module_segments.pop())
            continue
        else:
            break
    else:
        if last_error is not None and error_callback is not None:
            error_callback(last_error)
        return alternative

    module = sys.modules[module_name]
    nonexistent = object()
    for segment in reversed(remainder):
        module = getattr(module, segment, nonexistent)
        if module is nonexistent:
            if last_error is not None and error_callback is not None:
                error_callback(last_error)
            return alternative

    return module


def map_values(function, dictionary):
    """Map ``function`` across the values of ``dictionary``.

    :return: A dict with the same keys as ``dictionary``, where the value
        of each key ``k`` is ``function(dictionary[k])``.
    """
    return {k: function(dictionary[k]) for k in dictionary}


def filter_values(function, dictionary):
    """Filter ``dictionary`` by its values using ``function``."""
    return {k: v for k, v in dictionary.items() if function(v)}


def dict_subtract(a, b):
    """Return the part of ``a`` that's not in ``b``."""
    return {k: a[k] for k in set(a) - set(b)}


def list_subtract(a, b):
    """Return a list ``a`` without the elements of ``b``.

    If a particular value is in ``a`` twice and ``b`` once then the returned
    list then that value will appear once in the returned list.
    """
    a_only = list(a)
    for x in b:
        if x in a_only:
            a_only.remove(x)
    return a_only



# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

"""Extensions to the standard Python unittest library."""

__all__ = [
    "clone_test_with_new_id",
    "CopyStreamResult",
    "ConcurrentTestSuite",
    "ConcurrentStreamTestSuite",
    "DecorateTestCaseResult",
    "ErrorHolder",
    "ExpectedException",
    "ExtendedToOriginalDecorator",
    "ExtendedToStreamDecorator",
    "FixtureSuite",
    "iterate_tests",
    "MultipleExceptions",
    "MultiTestResult",
    "PlaceHolder",
    "run_test_with",
    "ResourcedToStreamDecorator",
    "Tagger",
    "TestCase",
    "TestByTestResult",
    "TestResult",
    "TestResultDecorator",
    "TextTestResult",
    "RunTest",
    "skip",
    "skipIf",
    "skipUnless",
    "StreamFailFast",
    "StreamResult",
    "StreamResultRouter",
    "StreamSummary",
    "StreamTagger",
    "StreamToDict",
    "StreamToExtendedDecorator",
    "StreamToQueue",
    "TestControl",
    "ThreadsafeForwardingResult",
    "TimestampingStreamResult",
    "try_import",
    "unique_text_generator",
    "version",
    "__version__",
]

from testtools.helpers import try_import
from testtools.matchers._impl import Matcher  # noqa: F401

from testtools.runtest import (
    MultipleExceptions,
    RunTest,
)
from testtools.testcase import (
    DecorateTestCaseResult,
    ErrorHolder,
    ExpectedException,
    PlaceHolder,
    TestCase,
    clone_test_with_new_id,
    run_test_with,
    skip,
    skipIf,
    skipUnless,
    unique_text_generator,
)
from testtools.testresult import (
    CopyStreamResult,
    ExtendedToOriginalDecorator,
    ExtendedToStreamDecorator,
    MultiTestResult,
    ResourcedToStreamDecorator,
    StreamFailFast,
    StreamResult,
    StreamResultRouter,
    StreamSummary,
    StreamTagger,
    StreamToDict,
    StreamToExtendedDecorator,
    StreamToQueue,
    Tagger,
    TestByTestResult,
    TestControl,
    TestResult,
    TestResultDecorator,
    TextTestResult,
    ThreadsafeForwardingResult,
    TimestampingStreamResult,
)
from testtools.testsuite import (
    ConcurrentTestSuite,
    ConcurrentStreamTestSuite,
    FixtureSuite,
    iterate_tests,
)

# same format as sys.version_info: "A tuple containing the five components of
# the version number: major, minor, micro, releaselevel, and serial. All
# values except releaselevel are integers; the release level is 'alpha',
# 'beta', 'candidate', or 'final'. The version_info value corresponding to the
# Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
# releaselevel of 'dev' for unreleased under-development code.
#
# If the releaselevel is 'alpha' then the major/minor/micro components are not
# established at this point, and setup.py will use a version of next-$(revno).
# If the releaselevel is 'final', then the tarball will be major.minor.micro.
# Otherwise it is major.minor.micro~$(revno).

try:
    # If setuptools_scm is installed (e.g. in a development environment with
    # an editable install), then use it to determine the version dynamically.
    from setuptools_scm import get_version

    # This will fail with LookupError if the package is not installed in
    # editable mode or if Git is not installed.
    version = get_version(root="..", relative_to=__file__)
    __version__ = tuple(version.split("."))
except (ImportError, LookupError):
    # As a fallback, use the version that is hard-coded in the file.
    try:
        from ._version import __version__, version
    except ModuleNotFoundError:
        # The user is probably trying to run this without having installed
        # the package, so complain.
        raise RuntimeError(
            "Testtools is not correctly installed. Please install it with pip."
        )



# Copyright (c) 2009-2016 testtools developers. See LICENSE for details.

__all__ = ["Warnings", "WarningMessage", "IsDeprecated"]

import warnings

from ._basic import Is
from ._const import Always
from ._datastructures import MatchesListwise, MatchesStructure
from ._higherorder import (
    AfterPreprocessing,
    Annotate,
)
from ._impl import Mismatch


def WarningMessage(category_type, message=None, filename=None, lineno=None, line=None):
    r"""
    Create a matcher that will match `warnings.WarningMessage`\s.

    For example, to match captured `DeprecationWarning`s with a message about
    some ``foo`` being replaced with ``bar``:

    .. code-block:: python

       WarningMessage(DeprecationWarning,
                      message=MatchesAll(
                          Contains('foo is deprecated'),
                          Contains('use bar instead')))

    :param type category_type: A warning type, for example `DeprecationWarning`.
    :param message_matcher: A matcher object that will be evaluated against
        warning's message.
    :param filename_matcher: A matcher object that will be evaluated against
        the warning's filename.
    :param lineno_matcher: A matcher object that will be evaluated against the
        warning's line number.
    :param line_matcher: A matcher object that will be evaluated against the
        warning's line of source code.
    """
    category_matcher = Is(category_type)
    message_matcher = message or Always()
    filename_matcher = filename or Always()
    lineno_matcher = lineno or Always()
    line_matcher = line or Always()
    return MatchesStructure(
        category=Annotate("Warning's category type does not match", category_matcher),
        message=Annotate(
            "Warning's message does not match", AfterPreprocessing(str, message_matcher)
        ),
        filename=Annotate("Warning's filname does not match", filename_matcher),
        lineno=Annotate("Warning's line number does not match", lineno_matcher),
        line=Annotate("Warning's source line does not match", line_matcher),
    )


class Warnings:
    """
    Match if the matchee produces warnings.
    """

    def __init__(self, warnings_matcher=None):
        """
        Create a Warnings matcher.

        :param warnings_matcher: Optional validator for the warnings emitted by
        matchee. If no warnings_matcher is supplied then the simple fact that
        at least one warning is emitted is considered enough to match on.
        """
        self.warnings_matcher = warnings_matcher

    def match(self, matchee):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            matchee()
            if self.warnings_matcher is not None:
                return self.warnings_matcher.match(w)
            elif not w:
                return Mismatch("Expected at least one warning, got none")

    def __str__(self):
        return f"Warnings({self.warnings_matcher!s})"


def IsDeprecated(message):
    """
    Make a matcher that checks that a callable produces exactly one
    `DeprecationWarning`.

    :param message: Matcher for the warning message.
    """
    return Warnings(
        MatchesListwise(
            [WarningMessage(category_type=DeprecationWarning, message=message)]
        )
    )



# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

import re

from testtools import TestCase
from testtools.compat import (
    text_repr,
    _b,
)
from testtools.matchers._basic import (
    _BinaryMismatch,
    Contains,
    DoesNotEndWith,
    DoesNotStartWith,
    EndsWith,
    Equals,
    Is,
    IsInstance,
    LessThan,
    GreaterThan,
    HasLength,
    MatchesRegex,
    NotEquals,
    SameMembers,
    StartsWith,
)
from testtools.tests.helpers import FullStackRunTest
from testtools.tests.matchers.helpers import TestMatchersInterface


class Test_BinaryMismatch(TestCase):
    """Mismatches from binary comparisons need useful describe output"""

    _long_string = "This is a longish multiline non-ascii string\n\xa7"
    _long_b = _b(_long_string)
    _long_u = _long_string

    class CustomRepr:
        def __init__(self, repr_string):
            self._repr_string = repr_string

        def __repr__(self):
            return "<object " + self._repr_string + ">"

    def test_short_objects(self):
        o1, o2 = self.CustomRepr("a"), self.CustomRepr("b")
        mismatch = _BinaryMismatch(o1, "!~", o2)
        self.assertEqual(mismatch.describe(), f"{o1!r} !~ {o2!r}")

    def test_short_mixed_strings(self):
        b, u = _b("\xa7"), "\xa7"
        mismatch = _BinaryMismatch(b, "!~", u)
        self.assertEqual(mismatch.describe(), f"{b!r} !~ {u!r}")

    def test_long_bytes(self):
        one_line_b = self._long_b.replace(_b("\n"), _b(" "))
        mismatch = _BinaryMismatch(one_line_b, "!~", self._long_b)
        self.assertEqual(
            mismatch.describe(),
            "{}:\nreference = {}\nactual    = {}\n".format(
                "!~",
                text_repr(self._long_b, multiline=True),
                text_repr(one_line_b),
            ),
        )

    def test_long_unicode(self):
        one_line_u = self._long_u.replace("\n", " ")
        mismatch = _BinaryMismatch(one_line_u, "!~", self._long_u)
        self.assertEqual(
            mismatch.describe(),
            "{}:\nreference = {}\nactual    = {}\n".format(
                "!~",
                text_repr(self._long_u, multiline=True),
                text_repr(one_line_u),
            ),
        )

    def test_long_mixed_strings(self):
        mismatch = _BinaryMismatch(self._long_b, "!~", self._long_u)
        self.assertEqual(
            mismatch.describe(),
            "{}:\nreference = {}\nactual    = {}\n".format(
                "!~",
                text_repr(self._long_u, multiline=True),
                text_repr(self._long_b, multiline=True),
            ),
        )

    def test_long_bytes_and_object(self):
        obj = object()
        mismatch = _BinaryMismatch(self._long_b, "!~", obj)
        self.assertEqual(
            mismatch.describe(),
            "{}:\nreference = {}\nactual    = {}\n".format(
                "!~",
                repr(obj),
                text_repr(self._long_b, multiline=True),
            ),
        )

    def test_long_unicode_and_object(self):
        obj = object()
        mismatch = _BinaryMismatch(self._long_u, "!~", obj)
        self.assertEqual(
            mismatch.describe(),
            "{}:\nreference = {}\nactual    = {}\n".format(
                "!~",
                repr(obj),
                text_repr(self._long_u, multiline=True),
            ),
        )


class TestEqualsInterface(TestCase, TestMatchersInterface):
    matches_matcher = Equals(1)
    matches_matches = [1]
    matches_mismatches = [2]

    str_examples = [("Equals(1)", Equals(1)), ("Equals('1')", Equals("1"))]

    describe_examples = [
        ("2 != 1", 2, Equals(1)),
        (
            (
                "!=:\n"
                "reference = 'abcdefghijklmnopqrstuvwxyz0123456789'\n"
                "actual    = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'\n"
            ),
            "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
            Equals("abcdefghijklmnopqrstuvwxyz0123456789"),
        ),
    ]


class TestNotEqualsInterface(TestCase, TestMatchersInterface):
    matches_matcher = NotEquals(1)
    matches_matches = [2]
    matches_mismatches = [1]

    str_examples = [("NotEquals(1)", NotEquals(1)), ("NotEquals('1')", NotEquals("1"))]

    describe_examples = [("1 == 1", 1, NotEquals(1))]


class TestIsInterface(TestCase, TestMatchersInterface):
    foo = object()
    bar = object()

    matches_matcher = Is(foo)
    matches_matches = [foo]
    matches_mismatches = [bar, 1]

    str_examples = [("Is(2)", Is(2))]

    describe_examples = [("2 is not 1", 2, Is(1))]


class TestIsInstanceInterface(TestCase, TestMatchersInterface):
    class Foo:
        pass

    matches_matcher = IsInstance(Foo)
    matches_matches = [Foo()]
    matches_mismatches = [object(), 1, Foo]

    str_examples = [
        ("IsInstance(str)", IsInstance(str)),
        ("IsInstance(str, int)", IsInstance(str, int)),
    ]

    describe_examples = [
        ("'foo' is not an instance of int", "foo", IsInstance(int)),
        (
            "'foo' is not an instance of any of (int, type)",
            "foo",
            IsInstance(int, type),
        ),
    ]


class TestLessThanInterface(TestCase, TestMatchersInterface):
    matches_matcher = LessThan(4)
    matches_matches = [-5, 3]
    matches_mismatches = [4, 5, 5000]

    str_examples = [
        ("LessThan(12)", LessThan(12)),
    ]

    describe_examples = [
        ("5 >= 4", 5, LessThan(4)),
        ("4 >= 4", 4, LessThan(4)),
    ]


class TestGreaterThanInterface(TestCase, TestMatchersInterface):
    matches_matcher = GreaterThan(4)
    matches_matches = [5, 8]
    matches_mismatches = [-2, 0, 4]

    str_examples = [
        ("GreaterThan(12)", GreaterThan(12)),
    ]

    describe_examples = [
        ("4 <= 5", 4, GreaterThan(5)),
        ("4 <= 4", 4, GreaterThan(4)),
    ]


class TestContainsInterface(TestCase, TestMatchersInterface):
    matches_matcher = Contains("foo")
    matches_matches = ["foo", "afoo", "fooa"]
    matches_mismatches = ["f", "fo", "oo", "faoo", "foao"]

    str_examples = [
        ("Contains(1)", Contains(1)),
        ("Contains('foo')", Contains("foo")),
    ]

    describe_examples = [("1 not in 2", 2, Contains(1))]


class DoesNotStartWithTests(TestCase):
    run_tests_with = FullStackRunTest

    def test_describe(self):
        mismatch = DoesNotStartWith("fo", "bo")
        self.assertEqual("'fo' does not start with 'bo'.", mismatch.describe())

    def test_describe_non_ascii_unicode(self):
        string = "A\xa7"
        suffix = "B\xa7"
        mismatch = DoesNotStartWith(string, suffix)
        self.assertEqual(
            "{} does not start with {}.".format(text_repr(string), text_repr(suffix)),
            mismatch.describe(),
        )

    def test_describe_non_ascii_bytes(self):
        string = _b("A\xa7")
        suffix = _b("B\xa7")
        mismatch = DoesNotStartWith(string, suffix)
        self.assertEqual(
            f"{string!r} does not start with {suffix!r}.", mismatch.describe()
        )


class StartsWithTests(TestCase):
    run_tests_with = FullStackRunTest

    def test_str(self):
        matcher = StartsWith("bar")
        self.assertEqual("StartsWith('bar')", str(matcher))

    def test_str_with_bytes(self):
        b = _b("\xa7")
        matcher = StartsWith(b)
        self.assertEqual(f"StartsWith({b!r})", str(matcher))

    def test_str_with_unicode(self):
        u = "\xa7"
        matcher = StartsWith(u)
        self.assertEqual(f"StartsWith({u!r})", str(matcher))

    def test_match(self):
        matcher = StartsWith("bar")
        self.assertIs(None, matcher.match("barf"))

    def test_mismatch_returns_does_not_start_with(self):
        matcher = StartsWith("bar")
        self.assertIsInstance(matcher.match("foo"), DoesNotStartWith)

    def test_mismatch_sets_matchee(self):
        matcher = StartsWith("bar")
        mismatch = matcher.match("foo")
        self.assertEqual("foo", mismatch.matchee)

    def test_mismatch_sets_expected(self):
        matcher = StartsWith("bar")
        mismatch = matcher.match("foo")
        self.assertEqual("bar", mismatch.expected)


class DoesNotEndWithTests(TestCase):
    run_tests_with = FullStackRunTest

    def test_describe(self):
        mismatch = DoesNotEndWith("fo", "bo")
        self.assertEqual("'fo' does not end with 'bo'.", mismatch.describe())

    def test_describe_non_ascii_unicode(self):
        string = "A\xa7"
        suffix = "B\xa7"
        mismatch = DoesNotEndWith(string, suffix)
        self.assertEqual(
            "{} does not end with {}.".format(text_repr(string), text_repr(suffix)),
            mismatch.describe(),
        )

    def test_describe_non_ascii_bytes(self):
        string = _b("A\xa7")
        suffix = _b("B\xa7")
        mismatch = DoesNotEndWith(string, suffix)
        self.assertEqual(
            f"{string!r} does not end with {suffix!r}.", mismatch.describe()
        )


class EndsWithTests(TestCase):
    run_tests_with = FullStackRunTest

    def test_str(self):
        matcher = EndsWith("bar")
        self.assertEqual("EndsWith('bar')", str(matcher))

    def test_str_with_bytes(self):
        b = _b("\xa7")
        matcher = EndsWith(b)
        self.assertEqual(f"EndsWith({b!r})", str(matcher))

    def test_str_with_unicode(self):
        u = "\xa7"
        matcher = EndsWith(u)
        self.assertEqual(f"EndsWith({u!r})", str(matcher))

    def test_match(self):
        matcher = EndsWith("arf")
        self.assertIs(None, matcher.match("barf"))

    def test_mismatch_returns_does_not_end_with(self):
        matcher = EndsWith("bar")
        self.assertIsInstance(matcher.match("foo"), DoesNotEndWith)

    def test_mismatch_sets_matchee(self):
        matcher = EndsWith("bar")
        mismatch = matcher.match("foo")
        self.assertEqual("foo", mismatch.matchee)

    def test_mismatch_sets_expected(self):
        matcher = EndsWith("bar")
        mismatch = matcher.match("foo")
        self.assertEqual("bar", mismatch.expected)


class TestSameMembers(TestCase, TestMatchersInterface):
    matches_matcher = SameMembers([1, 1, 2, 3, {"foo": "bar"}])
    matches_matches = [
        [1, 1, 2, 3, {"foo": "bar"}],
        [3, {"foo": "bar"}, 1, 2, 1],
        [3, 2, 1, {"foo": "bar"}, 1],
        (2, {"foo": "bar"}, 3, 1, 1),
    ]
    matches_mismatches = [
        {1, 2, 3},
        [1, 1, 2, 3, 5],
        [1, 2, 3, {"foo": "bar"}],
        "foo",
    ]

    describe_examples = [
        (
            (
                "elements differ:\n"
                "reference = ['apple', 'orange', 'canteloupe', 'watermelon', 'lemon', 'banana']\n"
                "actual    = ['orange', 'apple', 'banana', 'sparrow', 'lemon', 'canteloupe']\n"
                ": \n"
                "missing:    ['watermelon']\n"
                "extra:      ['sparrow']"
            ),
            [
                "orange",
                "apple",
                "banana",
                "sparrow",
                "lemon",
                "canteloupe",
            ],
            SameMembers(
                [
                    "apple",
                    "orange",
                    "canteloupe",
                    "watermelon",
                    "lemon",
                    "banana",
                ]
            ),
        ),
    ]

    str_examples = [
        ("SameMembers([1, 2, 3])", SameMembers([1, 2, 3])),
    ]


class TestMatchesRegex(TestCase, TestMatchersInterface):
    matches_matcher = MatchesRegex("a|b")
    matches_matches = ["a", "b"]
    matches_mismatches = ["c"]

    str_examples = [
        ("MatchesRegex('a|b')", MatchesRegex("a|b")),
        ("MatchesRegex('a|b', re.M)", MatchesRegex("a|b", re.M)),
        ("MatchesRegex('a|b', re.I|re.M)", MatchesRegex("a|b", re.I | re.M)),
        ("MatchesRegex({!r})".format(_b("\xa7")), MatchesRegex(_b("\xa7"))),
        ("MatchesRegex({!r})".format("\xa7"), MatchesRegex("\xa7")),
    ]

    describe_examples = [
        ("'c' does not match /a|b/", "c", MatchesRegex("a|b")),
        ("'c' does not match /a\\d/", "c", MatchesRegex(r"a\d")),
        (
            "{!r} does not match /\\s+\\xa7/".format(_b("c")),
            _b("c"),
            MatchesRegex(_b("\\s+\xa7")),
        ),
        ("{!r} does not match /\\s+\\xa7/".format("c"), "c", MatchesRegex("\\s+\xa7")),
    ]


class TestHasLength(TestCase, TestMatchersInterface):
    matches_matcher = HasLength(2)
    matches_matches = [[1, 2]]
    matches_mismatches = [[], [1], [3, 2, 1]]

    str_examples = [
        ("HasLength(2)", HasLength(2)),
    ]

    describe_examples = [
        ("len([]) != 1", [], HasLength(1)),
    ]


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2008-2012 testtools developers. See LICENSE for details.

import doctest

from testtools import TestCase
from testtools.compat import (
    _b,
)
from testtools.matchers._doctest import DocTestMatches
from testtools.tests.helpers import FullStackRunTest
from testtools.tests.matchers.helpers import TestMatchersInterface


class TestDocTestMatchesInterface(TestCase, TestMatchersInterface):
    matches_matcher = DocTestMatches("Ran 1 test in ...s", doctest.ELLIPSIS)
    matches_matches = ["Ran 1 test in 0.000s", "Ran 1 test in 1.234s"]
    matches_mismatches = ["Ran 1 tests in 0.000s", "Ran 2 test in 0.000s"]

    str_examples = [
        (
            "DocTestMatches('Ran 1 test in ...s\\n')",
            DocTestMatches("Ran 1 test in ...s"),
        ),
        ("DocTestMatches('foo\\n', flags=8)", DocTestMatches("foo", flags=8)),
    ]

    describe_examples = [
        (
            "Expected:\n    Ran 1 tests in ...s\nGot:\n" "    Ran 1 test in 0.123s\n",
            "Ran 1 test in 0.123s",
            DocTestMatches("Ran 1 tests in ...s", doctest.ELLIPSIS),
        )
    ]


class TestDocTestMatchesInterfaceUnicode(TestCase, TestMatchersInterface):
    matches_matcher = DocTestMatches("\xa7...", doctest.ELLIPSIS)
    matches_matches = ["\xa7", "\xa7 more\n"]
    matches_mismatches = ["\\xa7", "more \xa7", "\n\xa7"]

    str_examples = [
        ("DocTestMatches({!r})".format("\xa7\n"), DocTestMatches("\xa7")),
    ]

    describe_examples = [
        (
            "Expected:\n    \xa7\nGot:\n    a\n",
            "a",
            DocTestMatches("\xa7", doctest.ELLIPSIS),
        )
    ]


class TestDocTestMatchesSpecific(TestCase):
    run_tests_with = FullStackRunTest

    def test___init__simple(self):
        matcher = DocTestMatches("foo")
        self.assertEqual("foo\n", matcher.want)

    def test___init__flags(self):
        matcher = DocTestMatches("bar\n", doctest.ELLIPSIS)
        self.assertEqual("bar\n", matcher.want)
        self.assertEqual(doctest.ELLIPSIS, matcher.flags)

    def test_describe_non_ascii_bytes(self):
        """Even with bytestrings, the mismatch should be coercible to unicode

        DocTestMatches is intended for text, but the Python 2 str type also
        permits arbitrary binary inputs. This is a slightly bogus thing to do,
        and under Python 3 using bytes objects will reasonably raise an error.
        """
        header = _b("\x89PNG\r\n\x1a\n...")
        self.assertRaises(TypeError, DocTestMatches, header, doctest.ELLIPSIS)


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2011 testtools developers. See LICENSE for details.


import sys

from testtools import (
    ExpectedException,
    TestCase,
)
from testtools.matchers import (
    AfterPreprocessing,
    Equals,
    EndsWith,
)


class TestExpectedException(TestCase):
    """Test the ExpectedException context manager."""

    def test_pass_on_raise(self):
        with ExpectedException(ValueError, "tes."):
            raise ValueError("test")

    def test_pass_on_raise_matcher(self):
        with ExpectedException(ValueError, AfterPreprocessing(str, Equals("test"))):
            raise ValueError("test")

    def test_raise_on_text_mismatch(self):
        try:
            with ExpectedException(ValueError, "tes."):
                raise ValueError("mismatch")
        except AssertionError:
            e = sys.exc_info()[1]
            self.assertEqual("'mismatch' does not match /tes./", str(e))
        else:
            self.fail("AssertionError not raised.")

    def test_raise_on_general_mismatch(self):
        matcher = AfterPreprocessing(str, Equals("test"))
        value_error = ValueError("mismatch")
        try:
            with ExpectedException(ValueError, matcher):
                raise value_error
        except AssertionError:
            e = sys.exc_info()[1]
            self.assertEqual(matcher.match(value_error).describe(), str(e))
        else:
            self.fail("AssertionError not raised.")

    def test_raise_on_error_mismatch(self):
        try:
            with ExpectedException(TypeError, "tes."):
                raise ValueError("mismatch")
        except ValueError:
            e = sys.exc_info()[1]
            self.assertEqual("mismatch", str(e))
        else:
            self.fail("ValueError not raised.")

    def test_raise_if_no_exception(self):
        try:
            with ExpectedException(TypeError, "tes."):
                pass
        except AssertionError:
            e = sys.exc_info()[1]
            self.assertEqual("TypeError not raised.", str(e))
        else:
            self.fail("AssertionError not raised.")

    def test_pass_on_raise_any_message(self):
        with ExpectedException(ValueError):
            raise ValueError("whatever")

    def test_annotate(self):
        def die():
            with ExpectedException(ValueError, msg="foo"):
                pass

        exc = self.assertRaises(AssertionError, die)
        self.assertThat(exc.args[0], EndsWith(": foo"))

    def test_annotated_matcher(self):
        def die():
            with ExpectedException(ValueError, "bar", msg="foo"):
                pass

        exc = self.assertRaises(AssertionError, die)
        self.assertThat(exc.args[0], EndsWith(": foo"))


def test_suite():
    from unittest import TestLoader

    return TestLoader().loadTestsFromName(__name__)
